class RubyTokenParser

  # ========================================================================= #
  # === RubyTokenParser.parse
  #
  # The RubyTokenParser.parse() method will parse a String, and return
  # the (ruby) object which it contains.
  #
  # @usage example:
  #
  #   RubyTokenParser.parse(":foo") # => :foo
  #
  # @param [String] string
  #
  #   The (input) String which should be parsed.
  #
  # @param [nil, Hash] options
  #
  #   An options-hash
  #
  # @param [Boolean] do_raise_an_exception
  #
  # This boolean will determine whether we will raise an exception
  # or whether we will not.
  #
  # @option options [Boolean] :use_big_decimal
  #
  #   Whether to use BigDecimal instead of Float for objects like "1.23".
  #   Defaults to false.
  #
  # @option options [Boolean] :constant_base
  #
  #   Determines from what constant other constants are searched.
  #   Defaults to Object (nil is treated as Object too, Object
  #   is the toplevel-namespace).
  #
  # @return [Object] The object in the string.
  #
  # @raise [RubyTokenParser::SyntaxError]
  #
  #   If the String does not contain exactly one valid literal,
  #   a SyntaxError is raised.
  #
  # Usage example:
  #   x = RubyTokenParser.parse("[1,2,3]")
  # ========================================================================= #
  def self.parse(
      string,
      options = nil,
      do_raise_an_exception = RubyTokenParser.raise_exception?
    )
    # ======================================================================= #
    # === Instantiate a new parser
    # ======================================================================= #
    parser  = new(string, options)
    begin
      value = parser.scan_value
    rescue RubyTokenParser::SyntaxError # Must rescue things such as: @foo = foo
      value = RubyTokenParser::SyntaxError
    end
    if do_raise_an_exception
      unless parser.end_of_string? or
             value.nil?
        # =================================================================== #
        # Raise the Syntax Error.
        # =================================================================== #
        raise SyntaxError,
              "Unexpected superfluous data: #{parser.rest.inspect}"
      end unless value.is_a? Range # Make an exception for Range objects.
    end
    value
  end; self.instance_eval { alias [] parse } # === RubyTokenParser[]
       # ^^^ Add an alias to it, aka [].
end