class RubyTokenParser

# ========================================================================= #
# === RubyTokenParser::Expressions
#
# @private
#
# All the expressions used to parse the literals will be bundled here.
# ========================================================================= #
module Expressions # === RubyTokenParser::Expressions

  # ======================================================================= #
  # === Match to the start of an Array
  # ======================================================================= #
  RArrayBegin     = /\[/                                      

  # ======================================================================= #
  # === Match whitespace between elements in an Array
  # ======================================================================= #
  RArrayVoid      = /\s*/

  # ======================================================================= #
  # === Match the separator of Array elements
  #
  # This depends on RArrayVoid having been defined before.
  #
  # A separator in an Array is something such as:
  #   [1,2,3]
  # ======================================================================= #
  RArraySeparator = /#{RArrayVoid},#{RArrayVoid}/

  # ======================================================================= #
  # === Match to the end of an Array
  # ======================================================================= #
  RArrayEnd       = /\]/

  # ======================================================================= #
  # === Match to the start of a Hash
  # ======================================================================= #
  RHashBegin      = /\{/

  # ======================================================================= #
  # === Match whitespace between elements in a Hash
  # ======================================================================= #
  RHashVoid       = /\s*/

  # ======================================================================= #
  # === Match the separator of hash key/value pairs
  # ======================================================================= #
  RHashSeparator  = /#{RHashVoid},#{RHashVoid}/

  # ======================================================================= #
  # === Match the separator between a key and a value in a hash
  # ======================================================================= #
  RHashArrow      = /#{RHashVoid}=>#{RHashVoid}/

  # ======================================================================= #
  # === Match end of a Hash
  # ======================================================================= #
  RHashEnd        = /\}/

  # ======================================================================= #
  # === Match constant names (with nesting)
  # ======================================================================= #
  RConstant       = /[A-Z]\w*(?:::[A-Z]\w*)*/

  # ======================================================================= #
  # === Match to nil
  # ======================================================================= #
  RNil            = /nil/

  # ======================================================================= #
  # === Match false
  # ======================================================================= #
  RFalse          = /false/

  # ======================================================================= #
  # === Match true
  # ======================================================================= #
  RTrue           = /true/

  # ======================================================================= #
  # === Match an Integer in decimal notation
  # ======================================================================= #
  RInteger        = /[+-]?\d[\d_]*/

  # ======================================================================= #
  # === Match an Integer in binary notation
  # ======================================================================= #
  RBinaryInteger  = /[+-]?0b[01][01_]*/

  # ======================================================================= #
  # === Match an Integer in hexadecimal notation
  # ======================================================================= #
  RHexInteger     = /[+-]?0x[A-Fa-f\d][A-Fa-f\d_]*/

  # ======================================================================= #
  # === Match a decimal number (Float or BigDecimal)
  # ======================================================================= #
  RBigDecimal     = /#{RInteger}\.\d+/

  # ======================================================================= #
  # === Match a decimal number in scientific notation
  # ======================================================================= #
  RFloat          = /#{RBigDecimal}(?:f|e#{RInteger})/

  # ======================================================================= #
  # === Match a date
  # ======================================================================= #
  RDate           = /(\d{4})-(\d{2})-(\d{2})/

  # ======================================================================= #
  # === Match a time (without date)
  # ======================================================================= #
  RTime           = /(\d{2}):(\d{2}):(\d{2})(?:RTimeZone)?/

  # ======================================================================= #
  # === Match a datetime (must come after RTime was defined)
  # ======================================================================= #
  RDateTime       = /#{RDate}T#{RTime}/

  # ======================================================================= #
  # === Match a regular expression
  # ======================================================================= #
  RRegexp         = %r{/((?:[^\\/]+|\\.)*)/([imxnNeEsSuU]*)}

  # ======================================================================= #
  # === Match a double quoted string.
  # ======================================================================= #
  RDString        = /"(?:[^\\"]+|\\.)*"/

  # ======================================================================= #
  # === Match a single quoted string.
  # ======================================================================= #
  RSString        = /'(?:[^\\']+|\\.)*'/

  # ======================================================================= #
  # === Match a symbol                                         (symbol tag)
  #
  # Note that this depends on RSString and RDString, so these
  # must come before that.
  # ======================================================================= #
  RSymbol         = /:[A-Za-z_]\w*|:#{RSString}|:#{RDString}/

  # ======================================================================= #
  # === Match a symbol used as key in a Hash
  # ======================================================================= #
  RHashKeySymbol  = /([A-Za-z_]\w*):/

  # ======================================================================= #
  # === Match a timezone
  # ======================================================================= #
  RTimeZone       = /(Z|[A-Z]{3,4}|[+-]\d{4})/

  # ======================================================================= #
  # === Match a Regex such as (1..5)
  # ======================================================================= #
  RRange          = /\(\d*..\d*\)/

  # ======================================================================= #
  # === Match an Integer in octal notation
  # ======================================================================= #
  ROctalInteger   = /[+-]?0[0-7][0-7'_,]*/

  # ======================================================================= #
  # === DoubleQuotedStringEscapes
  #
  # Map escape sequences in double quoted strings.
  #
  # This will be a really huge Hash.
  # ======================================================================= #
  DoubleQuotedStringEscapes  = {
    '\\\\' => "\\",
    "\\'"  => "'",
    '\\"'  => '"',
    '\t'   => "\t",
    '\f'   => "\f",
    '\r'   => "\r",
    '\n'   => "\n",
  }
  256.times { |i|
    DoubleQuotedStringEscapes["\\%o"    % i] = i.chr
    DoubleQuotedStringEscapes["\\%03o"  % i] = i.chr
    DoubleQuotedStringEscapes["\\x%02x" % i] = i.chr
    DoubleQuotedStringEscapes["\\x%02X" % i] = i.chr
  }

end; end